/************************************************************************
* Rutoken                                                               *
* Copyright (C) Aktiv Co. 2003 - 2011                                   *
*  :  http://www.rutoken.ru                          *
*  :    http://www.rutoken.ru/hotline/download/drivers/*
*  : http://www.rutoken.ru/hotline/                 *
*-----------------------------------------------------------------------*
*        *
*    Rutoken    PKCS#11   C++:      *
*  -  ;                                          * 
*  -  ;                                        *
*  -   ;                                           *
*  -  ;                                               *
*  -  ;                                               *
*  -    .                           *
************************************************************************/
 

#include "PKCS11_Common.h"


/************************************************************************
*                    *
*  PKCS#11:                                                  *
*  -  ;                                              *
*  -  ;                                       *
*  -      PKCS#11;                      *
*  -   ;                                            *
*  -      ;                           *
*  -    ;                               *
*  -                      *
*         .             *
************************************************************************/
bool Init( IN  LPTSTR                 lpszPKCS11LibraryName, 
           IN  CK_FLAGS               ckFlags,
           IN  CK_USER_TYPE           userType,
           IN  const CK_UTF8CHAR_PTR  pPIN,
           IN  CK_ULONG               ulPINLength,
           OUT HMODULE&               hModule,
           OUT CK_SLOT_ID&            slotID,
           OUT CK_SESSION_HANDLE&     hSession,
           OUT CK_FUNCTION_LIST_PTR&  pFunctionList,
           OUT bool&                  bIsRutokenECP,
           OUT CK_RV&                 ckResult )
{
    bool                 bResult            = false;     //       
    bool                 bInitialized       = false;     //  ,    
    bool                 bLoggedIn          = false;     //  ,      

    CK_C_GetFunctionList pfnGetFunctionList = NULL_PTR;  //    C_GetFunctionList
        
    CK_SLOT_ID_PTR       aSlots             = NULL_PTR;  //     
    CK_ULONG             ulSlotCount        = 0;         //     
    
    hModule       = NULL_PTR;
    hSession      = NULL_PTR;
    pFunctionList = NULL_PTR;    

    PrintTextWithTab(TEXT("Initializing module...\n"));
    IncreaseTab(3);
    while(true)
    {
        /**********************************************************************
        *                                                  *      
        **********************************************************************/    
        PrintTextWithTab(TEXT("Loading library "));
        PrintString(lpszPKCS11LibraryName);
        PrintString(TEXT("...\n"));
        hModule = LoadLibrary(lpszPKCS11LibraryName);
        bResult = (hModule != NULL_PTR);
        if (!bResult)
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));

        /**********************************************************************
        *            *      
        **********************************************************************/            
        PrintTextWithTab(TEXT("Getting GetFunctionListFunction...\n"));
        pfnGetFunctionList = (CK_C_GetFunctionList)GetProcAddress(hModule,
                                                                  "C_GetFunctionList");
        bResult = (pfnGetFunctionList != NULL_PTR);
        if (!bResult)
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));

        /**********************************************************************
        *                               *      
        **********************************************************************/
        PrintTextWithTab(TEXT("Getting functions list...\n"));
        ckResult = pfnGetFunctionList(&pFunctionList);
        bResult = (ckResult == CKR_OK);
        if (!bResult)
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));

        /**********************************************************************
        *                                            *      
        **********************************************************************/
        PrintTextWithTab(TEXT("C_Initialize:...\n"));
        ckResult = pFunctionList->C_Initialize(NULL_PTR);
        bResult = (ckResult == CKR_OK);
        if(!bResult)
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));
        bInitialized = true;

        /**********************************************************************
        *    c                       *      
        **********************************************************************/        
        PrintTextWithTab(TEXT("Getting connected slots list. Step 1...\n"));
        ckResult = pFunctionList->C_GetSlotList(    CK_TRUE,
                                                    NULL_PTR,
                                                    &ulSlotCount ); 
        bResult = (ckResult == CKR_OK);
        if(!bResult)
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));

        aSlots = new CK_SLOT_ID[ulSlotCount];
        ::ZeroMemory(    aSlots,
                        (ulSlotCount * sizeof(CK_SLOT_ID)));

        PrintTextWithTab(TEXT("Getting connected slots list. Step 2...\n"));
        ckResult = pFunctionList->C_GetSlotList(    CK_TRUE,
                                                    aSlots,
                                                    &ulSlotCount ); 
        bResult = (ckResult == CKR_OK);
        if(!bResult)
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));

        if (ulSlotCount == 0)
        {
            PrintTextWithTab(TEXT("Error: no Rutoken available!\n"));
            bResult = false;
            break;
        }

        slotID = aSlots[0];

        PrintTextWithTab(TEXT("Slots available: "));
        PrintDword(ulSlotCount);
        PrintString(TEXT("\n"));

        /**********************************************************************
        *                                                *      
        **********************************************************************/        
        PrintTextWithTab(TEXT("Determining token type:...\n"));        
        bResult = GetTokenClass(    hModule,
                                    slotID,
                                    bIsRutokenECP,
                                    ckResult );        
        if(!bResult)
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));


        /**********************************************************************
        *                                   *      
        **********************************************************************/        
        PrintTextWithTab(TEXT("C_OpenSession:...\n"));
        ckResult = pFunctionList->C_OpenSession(    slotID,
                                                    ckFlags, 
                                                    NULL_PTR,
                                                    NULL_PTR,
                                                    &hSession );
        bResult = (ckResult == CKR_OK);
        if(!bResult)
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));

        if(    userType == CKU_SO
           ||  userType == CKU_USER)
        {
            /**********************************************************************
            *                            *      
            **********************************************************************/             
            PrintTextWithTab(TEXT("C_Login:...\n"));
            ckResult = pFunctionList->C_Login(  hSession,
                                                userType,
                                                pPIN,
                                                ulPINLength);
            bResult = (ckResult == CKR_OK);
            if(!bResult)
            {
                PrintTextWithTab(TEXT(" -> Failed\n"));
                break;
            }
            PrintTextWithTab(TEXT(" -> OK\n"));
            bLoggedIn = true;
        }
        else
        {
            PrintTextWithTab(TEXT("Using public session.\n"));
        }
        break;
    }
    DecreaseTab(3);
    if(aSlots)
    {
        delete[] aSlots;
        aSlots = NULL_PTR;
    }
    if(!bResult)
    {
        PrintTextWithTab(TEXT("Initialization failed!\n\n"));
        if(bLoggedIn)
        {
            pFunctionList->C_Logout(hSession);
        }
        if(hSession)
        {
            pFunctionList->C_CloseSession(hSession);
            hSession = NULL_PTR;
        }
        if(bInitialized)
        {
            pFunctionList->C_Finalize(NULL_PTR);
        }
        if(hModule)
        {
            FreeLibrary(hModule);
            hModule = NULL_PTR;
        }
        hSession = NULL_PTR;
        pFunctionList = NULL_PTR;
        hModule = NULL_PTR;
    }
    else
    {
        PrintTextWithTab(TEXT("Initialization has been completed successfully.\n\n"));
    }
    return bResult;
}

/************************************************************************
*                *
*  PKCS#11:                                                  *
*  -   ;                                            *
*  -    ;                                       *
*  -  ;                                     *
*  -    .                                    *
************************************************************************/
bool Free( IN  HMODULE&               hModule,
           IN  CK_SLOT_ID             slotID,
           IN  CK_SESSION_HANDLE&     hSession,
           IN  CK_FUNCTION_LIST_PTR&  pFunctionList,
           OUT CK_RV&                 ckResult )
{
    bool bResult = false;  //       
    
    PrintTextWithTab(TEXT("Finalizing...\n"));
    IncreaseTab(3);
    while(true)
    {
        if(hSession)
        {    
            /**********************************************************************
            *                                                 *
            **********************************************************************/
            PrintTextWithTab(TEXT("C_Logout...\n"));
            ckResult = pFunctionList->C_Logout(hSession);
            if(ckResult == CKR_USER_NOT_LOGGED_IN)
            {
                ckResult = CKR_OK;
            }
            bResult = (ckResult == CKR_OK);
            if(!bResult)
            {
                break;
            }
        
            /**********************************************************************
            *                                       *
            **********************************************************************/
            PrintTextWithTab(TEXT("C_CloseAllSession...\n"));
            ckResult = pFunctionList->C_CloseAllSessions(slotID);
            bResult = (ckResult == CKR_OK);
            if(!bResult)
            {
                break;
            }
            hSession = NULL_PTR;
        }

        if(pFunctionList)
        {    
            /**********************************************************************
            *                                         *
            **********************************************************************/
            PrintTextWithTab(TEXT("C_Finalize...\n"));
            ckResult = pFunctionList->C_Finalize(NULL_PTR);
            bResult = (ckResult == CKR_OK);
            if(!bResult)
            {
                break;
            }
            pFunctionList = NULL_PTR;
        }

        if(hModule)
        {    
            /**********************************************************************
            *                                          *
            **********************************************************************/
            PrintTextWithTab(TEXT("Unloading library...\n"));
            bResult = (FreeLibrary(hModule) == TRUE);
            if(!bResult)
            {
                break;
            }
            hModule = NULL_PTR;
        }

        bResult = true;
        break;
    }    
    if(pFunctionList)
    {
        pFunctionList->C_Finalize(NULL_PTR);
        pFunctionList =  NULL_PTR;
    }
    if(hModule)
    {
        FreeLibrary(hModule);
        hModule = NULL_PTR;
    }
    DecreaseTab(3);
    if(bResult)
    {
        PrintTextWithTab(TEXT("Unloading library has been completed successfully.\n\n"));
    }
    else
    {
        PrintTextWithTab(TEXT("Unloading library failed!\n\n"));
    }
    return bResult;
}

/************************************************************************
*   ,                      *
************************************************************************/
bool GetTokenClass( IN  HMODULE&    hModule, 
                    IN  CK_SLOT_ID  ulSlotID,
                    OUT bool&       bIsRutokenECP,
                    OUT CK_RV&      ckResult )
{
    bool                            bResult                    = true;      //       
    
    CK_C_EX_GetFunctionListExtended pfnGetFunctionListExtended = NULL_PTR;  //    C_EX_GetFunctionListExtended
    CK_FUNCTION_LIST_EXTENDED_PTR   pFunctionListExtended      = NULL_PTR;  //      PKCS#11,    CK_FUNCTION_LIST_EXTENDED    
    
    CK_TOKEN_INFO_EXTENDED          tokenInfoEx;                            //    CK_TOKEN_INFO_EXTENDED    
    
    while(true)
    {    
        /**********************************************************************
        *                     *      
        **********************************************************************/
        pfnGetFunctionListExtended = (CK_C_EX_GetFunctionListExtended)GetProcAddress(    hModule,
                                                                                        "C_EX_GetFunctionListExtended"    );
        bResult = (pfnGetFunctionListExtended !=  NULL_PTR);
        if (!bResult)
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }

        ckResult = pfnGetFunctionListExtended(&pFunctionListExtended);
        bResult = (ckResult == CKR_OK);
        if (!bResult)
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        
        ::ZeroMemory(&tokenInfoEx,
                     sizeof(CK_TOKEN_INFO_EXTENDED) );
        tokenInfoEx.ulSizeofThisStructure = sizeof(CK_TOKEN_INFO_EXTENDED);
                
        
        /**********************************************************************
        *                     *      
        **********************************************************************/
        ckResult = pFunctionListExtended->C_EX_GetTokenInfoExtended(    ulSlotID,
                                                                        &tokenInfoEx );
        bResult = (ckResult == CKR_OK);
        if(!bResult)
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        
        if(tokenInfoEx.ulTokenClass == TOKEN_CLASS_ECP )
        {
            bIsRutokenECP = true;
            PrintTextWithTab(TEXT("Rutoken ECP\n"));
        }
        else
        {
            bIsRutokenECP = false;
            PrintTextWithTab(TEXT("Rutoken S\n"));                                
        }        
        break;
    }
    return bResult;
}

/************************************************************************
*    CKA_VALUE                 *
************************************************************************/
bool GetObjectValue( IN  CK_SESSION_HANDLE     hSession,
                     IN  CK_FUNCTION_LIST_PTR  pFunctionList,
                     IN  CK_OBJECT_HANDLE      hObject,
                     OUT CK_BYTE_PTR&          pbtValue,
                     OUT CK_ULONG&             ulValueSize,
                     OUT CK_RV&                ckResult )
{
    bool         bResult   = false;                       //       

    CK_ATTRIBUTE attrValue = { CKA_VALUE,  NULL_PTR, 0};  //    CK_ATTRIBUTE     CKA_VALUE

    PrintTextWithTab(TEXT("Retrieving object value...\n"));
    IncreaseTab(3);    
    while(true)
    {    
        /**********************************************************************
        *        CKA_VALUE     *      
        **********************************************************************/
        PrintTextWithTab(TEXT("Getting object value. Step 1...\n"));
        ckResult = pFunctionList->C_GetAttributeValue(  hSession,
                                                        hObject,
                                                        &attrValue,
                                                        1 );
        bResult = (ckResult == CKR_OK);
        if(!bResult)
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));

        if(attrValue.ulValueLen == (CK_ULONG)-1)
        {
            PrintTextWithTab(TEXT("\nCannot obtain object value!\n"));
            bResult = false;
            break;
        }
            
        /**********************************************************************
        *               *      
        **********************************************************************/
        attrValue.pValue = new CK_BYTE[attrValue.ulValueLen];
        ::ZeroMemory(attrValue.pValue,
                     (attrValue.ulValueLen * sizeof(CK_BYTE)));
                

        /**********************************************************************
        *    CKA_VALUE                                *      
        **********************************************************************/
        PrintTextWithTab(TEXT("Getting object value. Step 2...\n"));
        ckResult = pFunctionList->C_GetAttributeValue(  hSession,
                                                        hObject,
                                                        &attrValue,
                                                        1 );
        bResult = (ckResult == CKR_OK);
        if(!bResult)
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));
        ulValueSize = attrValue.ulValueLen;
        pbtValue = new CK_BYTE[ulValueSize];
        ::ZeroMemory(pbtValue,
                     (ulValueSize * sizeof(CK_BYTE)));
        ::CopyMemory(pbtValue,
                     attrValue.pValue,
                     (ulValueSize * sizeof(CK_BYTE)));
        break;
    }
    if(attrValue.pValue)
    {
        delete[] attrValue.pValue;
        attrValue.pValue =  NULL_PTR;
        attrValue.ulValueLen = 0;
    }
    DecreaseTab(3);
    if(bResult)
    {
        PrintTextWithTab(TEXT("Object value has been obtained.\n\n"));
    }
    else
    {
        PrintTextWithTab(TEXT("Cannot obtain object value!\n\n"));
    }
    return bResult;
}

/************************************************************************
*    CKA_VALUE                            *
************************************************************************/
void SetValueToObjectTemplate( IN CK_ATTRIBUTE_PTR  pTemplate,
                               IN CK_ULONG          ulCount,
                               IN CK_BYTE_PTR       pbtValue,
                               IN CK_ULONG          ulValueSize)
{
    for(DWORD i = 0;
        i < ulCount;
        i++ )
    {
        if(pTemplate[i].type == CKA_VALUE)
        {
            pTemplate[i].pValue = pbtValue;
            pTemplate[i].ulValueLen = ulValueSize;
            break;
        }
    }
}

/************************************************************************
*    ,       *
************************************************************************/
bool FindObjects( IN  CK_SESSION_HANDLE      hSesion,
                  IN  CK_FUNCTION_LIST_PTR   pFunctionList,
                  IN  CK_ATTRIBUTE_PTR       pTemplates,
                  IN  CK_ULONG               ulCount,
                  OUT CK_OBJECT_HANDLE_PTR&  phObject,
                  OUT CK_ULONG&              ulObjectCount,
                  OUT CK_RV&                 ckResult )
{

    bool             bResult        = false;     //       
    bool             bFindInited    = false;     //  ,    

    CK_OBJECT_HANDLE hCurrentObject =  NULL_PTR; //     ,     
    DWORD            dwObjectsFound = 1;         //   -  ,    
    
    ulObjectCount = 0;
    phObject      = NULL_PTR;

    PrintTextWithTab(TEXT("Searching objects...\n"));
    IncreaseTab(3);    
    while(true)
    {        
        /**********************************************************************
        *                                       *      
        **********************************************************************/
        PrintTextWithTab(TEXT("C_FindObjectsInit:...\n"));
        ckResult = pFunctionList->C_FindObjectsInit(hSesion,
                                                    pTemplates,
                                                    ulCount);
        bResult = (ckResult == CKR_OK);
        if(!bResult)
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        bFindInited = true;
        PrintTextWithTab(TEXT(" -> OK\n"));

        /**********************************************************************
        *   ,                    *   
        * (   :      )     *     
        **********************************************************************/
        while(dwObjectsFound != 0)
        {
            hCurrentObject        = NULL_PTR;
            dwObjectsFound = 0;
            
            PrintTextWithTab(TEXT("C_FindObjects:...\n"));            
            ckResult = pFunctionList->C_FindObjects(hSesion,
                                                    &hCurrentObject,
                                                    1,
                                                    &dwObjectsFound );
            bResult = (ckResult == CKR_OK);
            if (!bResult)
            {
                PrintTextWithTab(TEXT(" -> Failed\n"));
                break;
            }
            PrintTextWithTab(TEXT(" -> OK\n"));
                                    
            if(dwObjectsFound != 0)
            {
                /**********************************************************************
                *                       *
                *                                                    *      
                **********************************************************************/
                CK_OBJECT_HANDLE_PTR aObjectsTemp =  NULL_PTR;
                if(ulObjectCount)
                {
                    aObjectsTemp = new CK_OBJECT_HANDLE[ulObjectCount];
                    ::ZeroMemory(aObjectsTemp,
                                 (ulObjectCount * sizeof(CK_OBJECT_HANDLE)));
                    for(DWORD i = 0;
                        i < ulObjectCount;
                        i++ )
                    {
                        aObjectsTemp[i] = phObject[i];
                    }
                }
                delete[] phObject;

                /**********************************************************************
                *      1.                          *                
                **********************************************************************/
                ulObjectCount ++;

                /**********************************************************************
                *   ,            *
                *  ,   +  ,   *
                *    ,                 *
                **********************************************************************/
                phObject = new CK_OBJECT_HANDLE[ulObjectCount];
                ::ZeroMemory(phObject,
                             (ulObjectCount * sizeof(CK_OBJECT_HANDLE)));                
                if(aObjectsTemp)
                {
                    /**********************************************************************
                    *              *
                    *      ,          *
                    **********************************************************************/
                    for(DWORD i = 0;
                        i < ulObjectCount-1;
                        i++)
                    {
                        phObject[i] = aObjectsTemp[i];
                    }
                    delete[] aObjectsTemp;
                    aObjectsTemp =  NULL_PTR;
                }
                phObject[ulObjectCount-1] = hCurrentObject;                
            }
        }
        break;
    }

    if(!bResult)
    {
        delete[] phObject;
        phObject =  NULL_PTR;
        ulObjectCount = 0;
    }

    if(bFindInited)
    {        
        /**********************************************************************
        *                                     *
        **********************************************************************/
        PrintTextWithTab(TEXT("C_FindObjectsFinal:...\n"));  
        ckResult = pFunctionList->C_FindObjectsFinal(hSesion);
        bResult = (ckResult == CKR_OK);
        if(!bResult)
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
        }
        else
        {
            PrintTextWithTab(TEXT(" -> OK\n"));
        }
    }
    DecreaseTab(3);

    if(bResult)
    {
        PrintTextWithTab(TEXT("Search has been completed.\n"));
        PrintTextWithTab(TEXT("Objects found:"));
        PrintDword(ulObjectCount);
        PrintString(TEXT("\n\n"));
    }
    else
    {
        PrintTextWithTab(TEXT("Search failed!\n\n"));
    }
    return bResult;
}

/**********************************************************************
*                                      *
**********************************************************************/
bool HashData( IN  CK_SESSION_HANDLE     hSession,
               IN  CK_FUNCTION_LIST_PTR  pFunctionList,
               IN  CK_MECHANISM_PTR      pHashMechanism,
               IN  CK_BYTE_PTR           pbtDataToHash,
               IN  CK_ULONG              ulDataToHashSize,
               OUT CK_BYTE_PTR&          pbtHash,
               OUT CK_ULONG&             ulHashSize,
               OUT CK_RV&                ckResult)
{
    bool bResult = false;                 //       
    
    pbtHash    = NULL_PTR;
    ulHashSize = 0;
    
    PrintTextWithTab(TEXT("Hashing data...\n"));
    IncreaseTab(3);
    while(true)
    {
        /**********************************************************************
        *                                  *
        **********************************************************************/
        PrintTextWithTab(TEXT("C_DigestInit...\n"));
        ckResult = pFunctionList->C_DigestInit(    hSession,
                                                pHashMechanism );
        bResult = (ckResult == CKR_OK);
        if(!bResult)
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));

        /**********************************************************************
        *                                      *
        **********************************************************************/
        PrintTextWithTab(TEXT("C_Digest step 1...\n"));
        ckResult = pFunctionList->C_Digest( hSession,
                                            pbtDataToHash,
                                            ulDataToHashSize,
                                            pbtHash,
                                            &ulHashSize );
        bResult = (ckResult == CKR_OK);
        if(!bResult)
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));

        pbtHash = new CK_BYTE[ulHashSize];
        ::ZeroMemory(pbtHash,
                     (ulHashSize* sizeof(CK_BYTE)));

        PrintTextWithTab(TEXT("C_Digest step 2...\n"));
        ckResult = pFunctionList->C_Digest( hSession,
                                            pbtDataToHash,
                                            ulDataToHashSize,
                                            pbtHash,
                                            &ulHashSize );
        bResult = (ckResult == CKR_OK);
        if(!bResult)
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));
        
        /************************************************************************
        *  ,                        *        
        ************************************************************************/
        PrintTextWithTab(TEXT("Hashed buffer is:\n"));
        PrintBuffer(pbtHash,
                    ulHashSize );
        break;
    }
    if(!bResult)
    {
        if(pbtHash)
        {
            delete[] pbtHash;
            pbtHash = NULL_PTR;
            ulHashSize = 0;
        }
    }
    DecreaseTab(3);
    if(bResult)
    {
        PrintTextWithTab(TEXT("Hashing has been completed.\n"));
    }
    else
    {
        PrintTextWithTab(TEXT("\n\nHashing failed!\n\n"));
    }
    return bResult;
}

/**********************************************************************
*                                 *
**********************************************************************/
bool SignData( IN  CK_SESSION_HANDLE    hSession,
               IN  CK_FUNCTION_LIST_PTR pFunctionList,
               IN  CK_ATTRIBUTE_PTR     pSignerKeyFindTemplate,
               IN  CK_ULONG             ulSignerKeyFindTemplateSize,
               IN  CK_MECHANISM_PTR     pHashMechanism,
               IN  CK_MECHANISM_PTR     pSignatureMechanism,
               IN  CK_BYTE_PTR          pbtDataToSign,
               IN  CK_ULONG             ulDataToSignSize,
               OUT CK_BYTE_PTR&         pbtSignature,
               OUT CK_ULONG&            ulSignatureSize,
               OUT CK_RV&               ckResult )
{
    bool                 bResult            = false;    //           

    CK_OBJECT_HANDLE_PTR ahSignerKeys       = NULL_PTR; //         
    CK_ULONG             ulSignerKeysNumber = 0;        //       

    CK_BYTE_PTR          pbHash             = NULL_PTR; //       
    CK_ULONG             ulHashSize         = 0;        //              

    pbtSignature    = NULL_PTR;
    ulSignatureSize = 0;

    PrintTextWithTab(TEXT("Signing data...\n"));
    IncreaseTab(3);
    while(true)
    {
        /************************************************************************
        *                                    *
        ************************************************************************/
        PrintTextWithTab(TEXT("Getting signature key...\n")); 
        bResult = FindObjects(  hSession,
                                pFunctionList,
                                pSignerKeyFindTemplate,
                                ulSignerKeyFindTemplateSize,
                                ahSignerKeys,
                                ulSignerKeysNumber,
                                ckResult );
        if(!bResult)
        {
            break;
        }
        if(ulSignerKeysNumber == 0)
        {
            PrintTextWithTab(TEXT("No signature key found!!!\n"));
            bResult = false;
            break;
        }

        /**********************************************************************
        *                                      *
        **********************************************************************/
        bResult = HashData( hSession,
                            pFunctionList,
                            pHashMechanism,
                            pbtDataToSign,
                            ulDataToSignSize,
                            pbHash,
                            ulHashSize,
                            ckResult);
        if(!bResult)
        {
            break;
        }

        /**********************************************************************
        *                                *
        **********************************************************************/
        PrintTextWithTab(TEXT("C_SignInit...\n"));
        ckResult = pFunctionList->C_SignInit(   hSession,
                                                pSignatureMechanism,
                                                ahSignerKeys[0]);
        bResult = (ckResult == CKR_OK);
        if(!bResult)
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));

        /**********************************************************************
        *                                              *
        **********************************************************************/
        PrintTextWithTab(TEXT("C_Sign step 1...\n"));
        ckResult = pFunctionList->C_Sign(   hSession,
                                            pbHash,
                                            ulHashSize,
                                            pbtSignature,
                                            &ulSignatureSize );
        bResult = (ckResult == CKR_OK);
        if(!bResult)
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));

        pbtSignature = new CK_BYTE[ulSignatureSize];
        ZeroMemory( pbtSignature,
                    ulSignatureSize* sizeof(CK_BYTE));
        PrintTextWithTab(TEXT("C_Sign step 2...\n"));
        ckResult = pFunctionList->C_Sign(   hSession,
                                            pbHash,
                                            ulHashSize,
                                            pbtSignature,
                                            &ulSignatureSize );
        bResult = (ckResult == CKR_OK);
        if(!bResult)
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));

        /************************************************************************
        *  ,                                   *        
        ************************************************************************/
        PrintTextWithTab(TEXT("Signature buffer is:\n"));
        PrintBuffer(    pbtSignature,
                        ulSignatureSize );
        break;
    }
    if(ahSignerKeys)
    {
        delete[] ahSignerKeys;
        ahSignerKeys = NULL_PTR;
    }
    if(pbHash)
    {
        delete[] pbHash;
        pbHash = NULL_PTR;
    }
    if(!bResult)
    {
        if(pbtSignature)
        {
            delete[] pbtSignature;
            pbtSignature = NULL_PTR;
            ulSignatureSize = 0;
        }
    }
    DecreaseTab(3);
    if(bResult)
    {
        PrintTextWithTab(TEXT("Data has been signed successfully.\n\n"));
    }
    else
    {
        PrintTextWithTab(TEXT("Signing operation failed!\n\n"));
    }
    return bResult;
}

/**********************************************************************
*                                    *
**********************************************************************/
bool VerifySignature( IN  CK_SESSION_HANDLE    hSession,
                      IN  CK_FUNCTION_LIST_PTR pFunctionList,
                      IN  CK_ATTRIBUTE_PTR     pVerifierKeyFindTemplate,
                      IN  CK_ULONG             ulVerifierKeyFindTemplateSize,
                      IN  CK_MECHANISM_PTR     pHashMechanism,
                      IN  CK_MECHANISM_PTR     pSignatureMechanism,
                      IN  CK_BYTE_PTR          pbtDataToSign,
                      IN  CK_ULONG             ulDataToSignSize,
                      IN  CK_BYTE_PTR          pbtSignature, 
                      IN  CK_ULONG             ulSignatureSize,
                      OUT CK_RV&               ckResult)
{    
    bool                 bResult              = false;    //           

    CK_OBJECT_HANDLE_PTR ahVerifierKeys       = NULL_PTR; //         
    CK_ULONG             ulVerifierKeysNumber = 0;        //       

    CK_BYTE_PTR          pbHash               = NULL_PTR; //       
    CK_ULONG             ulHashSize           = 0;        //          
    

    PrintTextWithTab(TEXT("Verifying signature...\n"));
    IncreaseTab(3);
    while(true)
    {
        /************************************************************************
        *                                  *
        ************************************************************************/
        PrintTextWithTab(TEXT("Getting verification key...\n")); 
        bResult = FindObjects(  hSession,
                                pFunctionList,
                                pVerifierKeyFindTemplate,
                                ulVerifierKeyFindTemplateSize,
                                ahVerifierKeys,
                                ulVerifierKeysNumber,
                                ckResult );
        if(!bResult)
        {
            break;
        }
        if(ulVerifierKeysNumber == 0)
        {
            PrintTextWithTab(TEXT("No verification key found!!!\n"));
            bResult = false;
            break;
        }

        /**********************************************************************
        *                                      *
        **********************************************************************/
        bResult = HashData( hSession,
                            pFunctionList,
                            pHashMechanism,
                            pbtDataToSign,
                            ulDataToSignSize,
                            pbHash,
                            ulHashSize,
                            ckResult);
        if(!bResult)
        {
            break;
        }

        /**********************************************************************
        *                              *
        **********************************************************************/
        PrintTextWithTab(TEXT("C_VerifyInit...\n"));
        ckResult = pFunctionList->C_VerifyInit( hSession,
                                                pSignatureMechanism,
                                                ahVerifierKeys[0]);
        bResult = (ckResult == CKR_OK);
        if(!bResult)
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));


        /**********************************************************************
        *                                    *
        **********************************************************************/
        PrintTextWithTab(TEXT("C_Verify...\n"));
        ckResult = pFunctionList->C_Verify( hSession,
                                            pbHash,
                                            ulHashSize,
                                            pbtSignature,
                                            ulSignatureSize );
        bResult = (ckResult == CKR_OK);
        if(!bResult)
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));

        break;
    }
    if(ahVerifierKeys)
    {
        delete[] ahVerifierKeys;
        ahVerifierKeys = NULL_PTR;
    }
    if(pbHash)
    {
        delete[] pbHash;
        pbHash = NULL_PTR;
    }
    DecreaseTab(3);
    
    if(bResult)
    {
        PrintTextWithTab(TEXT("Verifying has been completed successfully.\n\n"));
    }
    else
    {
        PrintTextWithTab(TEXT("Verifying failed!\n\n"));
    }
    return bResult;
}