/************************************************************************
* Rutoken                                                               *
* Copyright (C) Aktiv Co. 2003 - 2011                                   *
*  :  http://www.rutoken.ru                          *
*  :    http://www.rutoken.ru/hotline/download/drivers/*
*  : http://www.rutoken.ru/hotline/                 *
*-----------------------------------------------------------------------*
*    Rutoken    PKCS#11   C++    *
*-----------------------------------------------------------------------*
*        Rutoken:    *
*  -    Rutoken    ;        *
*  -     Rutoken (Login);         *
*  -   RSA;                                                *
*  -      Rutoken (Logout)       *
*      Rutoken.                                              *
*-----------------------------------------------------------------------*
*                        *
*   PKCS#11.                                                *
*              *
*    PKCS#11.                                         *
************************************************************************/

/************************************************************************
*   PKCS11_Common.h.                                      *
*  PKCS11_Common.h  **     :  *
*  -   :                                     *
      - PKCS#11;                                                        *
	  -  /;                                      *  
*     -  Win32 API;                                              * 
*  -  DEMO-;                                             *  
*  -  DEMO PIN- ;                                *
*  -  ,    .      *
************************************************************************/
    

#include "PKCS11_Common.h"


/*     */
CK_OBJECT_CLASS     ocPubKey    = CKO_PUBLIC_KEY;
CK_OBJECT_CLASS     ocPrivKey   = CKO_PRIVATE_KEY;
CK_KEY_TYPE         ktRSA       = CKK_RSA;
CK_BBOOL            ckbTrue_    = CK_TRUE;
CK_BBOOL            ckbFalse_   = CK_FALSE;

/*    RSA  */
CK_ULONG    ulRSAModBits = 512;

/************************************************************************
*      RSA                               * 
* (     )                              *
************************************************************************/
CK_ATTRIBUTE attrRSAExchPublicKeyTmpl[] = 
{
    { CKA_CLASS,        &ocPubKey,                         sizeof(ocPubKey)},
    { CKA_LABEL,        (CK_VOID_PTR)PUBLIC_RSA_KEY_LABEL, (CK_ULONG)sizeof(PUBLIC_RSA_KEY_LABEL)-1},
    { CKA_ID,           (CK_VOID_PTR)RSA_KEY_PAIR_ID,      (CK_ULONG)sizeof(RSA_KEY_PAIR_ID)-1},
    { CKA_KEY_TYPE,     &ktRSA,                            sizeof(ktRSA)},
    { CKA_TOKEN,        &ckbTrue_,                         sizeof(ckbTrue_)},
    { CKA_ENCRYPT,      &ckbTrue_,                         sizeof(ckbTrue_)},
    { CKA_PRIVATE,      &ckbFalse_,                        sizeof(ckbFalse_)},
    { CKA_MODULUS_BITS, &ulRSAModBits,                     sizeof(ulRSAModBits)}
};

/************************************************************************
*      RSA                               * 
* (     )                              *
************************************************************************/
CK_ATTRIBUTE attrRSAExchPrivateKeyTmpl[] = 
{
    { CKA_CLASS,    &ocPrivKey,                         sizeof(ocPrivKey)},
    { CKA_LABEL,    (CK_VOID_PTR)PRIVATE_RSA_KEY_LABEL, (CK_ULONG)sizeof(PRIVATE_RSA_KEY_LABEL)-1},
    { CKA_ID,       (CK_VOID_PTR)RSA_KEY_PAIR_ID,       (CK_ULONG)sizeof(RSA_KEY_PAIR_ID)-1},
    { CKA_KEY_TYPE, &ktRSA,                             sizeof(ktRSA)},
    { CKA_DECRYPT,  &ckbTrue_,                          sizeof(ckbTrue_)},
    { CKA_TOKEN,    &ckbTrue_,                          sizeof(ckbTrue_)},
    { CKA_PRIVATE,  &ckbTrue_,                          sizeof(ckbTrue_)}
};

/*      RSA */
CK_MECHANISM ckmRSAKeyGenMech = {CKM_RSA_PKCS_KEY_PAIR_GEN, NULL_PTR, 0};


/************************************************************************
* main()                                                                *
************************************************************************/
int main(int argc, char* argv[])
{
    HMODULE               hModulePKCS11      = NULL_PTR; //    PKCS#11
    CK_SLOT_ID            slotID             = 0;        //  ,       
    CK_SESSION_HANDLE     hSession           = NULL_PTR; //   
    CK_FUNCTION_LIST_PTR  pFunctionList      = NULL_PTR; //     PKCS#11,    CK_FUNCTION_LIST

    CK_OBJECT_HANDLE      hRSAExchPublicKey  = NULL_PTR; //    RSA (     )
    CK_OBJECT_HANDLE      hRSAExchPrivateKey = NULL_PTR; //    RSA (     )
    
    bool                  bResult            = false;    //       
    CK_RV                 ckResult           = CKR_OK;   //                          

    bool                  bIsRutokenECP      = false;    //       .                     

    while(true)
    {
        /**********************************************************************
        *  1:        PKCS#11.  *
        **********************************************************************/
        bResult = Init( PKCS11_LIBRARY_NAME,
                        CKF_SERIAL_SESSION | CKF_RW_SESSION,
                        CKU_USER,
                        (const CK_UTF8CHAR_PTR)USER_PIN,
                        (CK_ULONG)sizeof(USER_PIN),
                        hModulePKCS11,
                        slotID,
                        hSession,
                        pFunctionList,
                        bIsRutokenECP,
                        ckResult);
        if(!bResult)
        {
            break;
        }

        /**********************************************************************
        *  2:    RSA.                             *
        **********************************************************************/
        PrintTextWithTab(TEXT("Generating RSA exchange key pair...\n"));
        PrintTextWithTab(TEXT("C_GenerateKeyPair:...\n"));
        ckResult = pFunctionList->C_GenerateKeyPair(    hSession,
                                                        &ckmRSAKeyGenMech,
                                                        attrRSAExchPublicKeyTmpl,
                                                        arraysize(attrRSAExchPublicKeyTmpl),
                                                        attrRSAExchPrivateKeyTmpl,
                                                        arraysize(attrRSAExchPrivateKeyTmpl),
                                                        &hRSAExchPublicKey,
                                                        &hRSAExchPrivateKey );
        bResult = (ckResult == CKR_OK);
        if(!bResult)
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));
        break;
    }

    /**********************************************************************
    *  3:                          *
    *          PKCS#11.                                       *
    **********************************************************************/
    CK_RV ckTempResult = CKR_OK;
    Free(   hModulePKCS11,
            slotID,
            hSession,
            pFunctionList,
            ckTempResult);

    if(!bResult)
    {
        PrintLastError( TEXT("Some error occurred."),
                        ckResult);
    }
    else
    {
        PrintDone();
    }
    return bResult;
}

