/************************************************************************
* Rutoken                                                               *
* Copyright (C) Aktiv Co. 2003 - 2011                                   *
*  :  http://www.rutoken.ru                          *
*  :    http://www.rutoken.ru/hotline/download/drivers/*
*  : http://www.rutoken.ru/hotline/                 *
*-----------------------------------------------------------------------*
*    Rutoken    PKCS#11   C++    *
*-----------------------------------------------------------------------*
*      ""              * 
*  PKCS#11:                                                    *
*  -    Rutoken    ;        *
*  -   ;                                   *
*  -  PIN- ;                                *
*  -  PIN- ;                             *
*  -    ;                                        *
*  -    .                                         *
*-----------------------------------------------------------------------*
*               *
*   PKCS#11                                                 *
************************************************************************/

/************************************************************************
*   PKCS11_Common.h.                                      *
*  PKCS11_Common.h  **     :  *
*  -   :                                     *
      - PKCS#11;                                                        *
	  -  /;                                      *  
*     -  Win32 API;                                              * 
*  -  DEMO-;                                             *  
*  -  DEMO PIN- ;                                *
*  -  ,    .      *
************************************************************************/
    

#include "PKCS11_Common.h"


/*       */
#define MAX_ADMIN_RETRY_COUNT               10 

/*         */
#define MAX_USER_RETRY_COUNT                10 


/**********************************************************************
*                     *
**********************************************************************/
bool PrintExtendedTokenInfo( IN  CK_SLOT_ID                     slotID,                 //  ,       
                             IN  CK_FUNCTION_LIST_EXTENDED_PTR  pFunctionListExtended,  //      PKCS#11,    CK_FUNCTION_LIST_EXTENDED    
                             OUT CK_RV&                         ckResult                //  .     ,   PKCS#11    
                           )
{
    bool                   bResult                = false;  //       
    CK_TOKEN_INFO_EXTENDED tokenInfo_Ex;                    //    CK_TOKEN_INFO_EXTENDED    

    PrintTextWithTab(TEXT("Printing extended token info...\n"));
    IncreaseTab(3);    
    while(true)
    {
        /************************************************************************
        *     CK_TOKEN_INFO_EXTENDED                 *
        *                                                    *
        ************************************************************************/
        PrintTextWithTab(TEXT("Getting extended token info...\n"));
        PrintTextWithTab(TEXT("C_EX_GetTokenInfoExtended:...\n"));
        
        tokenInfo_Ex.ulSizeofThisStructure = sizeof(CK_TOKEN_INFO_EXTENDED);
        ckResult = pFunctionListExtended->C_EX_GetTokenInfoExtended( slotID,
                                                                     &tokenInfo_Ex);
        bResult = (ckResult == CKR_OK);
        if(!bResult)
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));
        
        /************************************************************************
        *                                           *        
        ************************************************************************/
        PrintTextWithTab(TEXT("Extended information:\n"));
        IncreaseTab(3);

        PrintTextWithTab(TEXT("Token type:               "));
        PrintDword(tokenInfo_Ex.ulTokenType);
        if(tokenInfo_Ex.ulTokenType == TOKEN_TYPE_RUTOKEN_ECP)
        {
            PrintString(TEXT("(Rutoken ECP)"));
        }
        else if(tokenInfo_Ex.ulTokenType == TOKEN_TYPE_RUTOKEN_LITE)
        {
            PrintString(TEXT("(Rutoken LITE)"));
        }
        else if(tokenInfo_Ex.ulTokenType == TOKEN_TYPE_RUTOKEN)
        {
            PrintString(TEXT("(Rutoken)"));
        }
        PrintString(TEXT("\n"));

        PrintTextWithTab(TEXT("Protocol number:          "));
        PrintDword(tokenInfo_Ex.ulProtocolNumber);
        PrintString(TEXT("\n"));

        PrintTextWithTab(TEXT("Microcode number:         "));
        PrintDword(tokenInfo_Ex.ulMicrocodeNumber);
        PrintString(TEXT("\n"));

        PrintTextWithTab(TEXT("Flags:                    "));
        PrintDword(tokenInfo_Ex.flags);
        PrintString(TEXT("\n"));

        PrintTextWithTab(TEXT("Max Admin PIN Len:        "));
        PrintDword(tokenInfo_Ex.ulMaxAdminPinLen);
        PrintString(TEXT("\n"));

        PrintTextWithTab(TEXT("Min Admin PIN Len:        "));
        PrintDword(tokenInfo_Ex.ulMinAdminPinLen);
        PrintString(TEXT("\n"));

        PrintTextWithTab(TEXT("Max User PIN Len:         "));
        PrintDword(tokenInfo_Ex.ulMaxUserPinLen);
        PrintString(TEXT("\n"));

        PrintTextWithTab(TEXT("Min User PIN Len:         "));
        PrintDword(tokenInfo_Ex.ulMinUserPinLen);
        PrintString(TEXT("\n"));

        PrintTextWithTab(TEXT("Max Admin retry counter:  "));
        PrintDword(tokenInfo_Ex.ulMaxAdminRetryCount);
        PrintString(TEXT("\n"));

        PrintTextWithTab(TEXT("Admin retry counter:      "));
        PrintDword(tokenInfo_Ex.ulAdminRetryCountLeft);
        PrintString(TEXT("\n"));

        PrintTextWithTab(TEXT("Max User retry counter:   "));
        PrintDword(tokenInfo_Ex.ulMaxUserRetryCount);
        PrintString(TEXT("\n"));

        PrintTextWithTab(TEXT("User retry counter:       "));
        PrintDword(tokenInfo_Ex.ulUserRetryCountLeft);
        PrintString(TEXT("\n"));

        PrintTextWithTab(TEXT("Serial number:\n"));
        PrintBuffer( tokenInfo_Ex.serialNumber, 
                     arraysize(tokenInfo_Ex.serialNumber) );

        PrintTextWithTab(TEXT("Total memory:             "));
        PrintDword(tokenInfo_Ex.ulTotalMemory);
        PrintString(TEXT("\n"));

        PrintTextWithTab(TEXT("Free memory:              "));
        PrintDword(tokenInfo_Ex.ulFreeMemory);
        PrintString(TEXT("\n"));

        PrintTextWithTab(TEXT("ATR:\n"));
        PrintBuffer( tokenInfo_Ex.ATR, 
                     tokenInfo_Ex.ulATRLen );
        DecreaseTab(3);

        break;
    }
    DecreaseTab(3);
    if(bResult)
    {
        PrintTextWithTab(TEXT("Extended info test has been completed successfully.\n"));
    }
    else
    {
        PrintTextWithTab(TEXT("Extended info test failed!\n"));
    }
    return bResult;
}

/************************************************************************
*     PIN-:                    *
*  -   PIN-                   * 
*     PIN-;                                               *
*  -     ;                 *
*  -  PIN- ;                               *
*  -   .                                            *
************************************************************************/
bool PerformExtendedPINTest( IN  CK_SESSION_HANDLE               hSession,               //  
                             IN  CK_FUNCTION_LIST_PTR            pFunctionList,          //     PKCS#11,    CK_FUNCTION_LIST
                             IN  CK_FUNCTION_LIST_EXTENDED_PTR   pFunctionListExtended,  //      PKCS#11,    CK_FUNCTION_LIST_EXTENDED    
                             OUT CK_RV&                          ckResult                //  .     ,   PKCS#11
                           )
{
    bool bResult = false;
    
    PrintTextWithTab(TEXT("Performing extended PIN function test...\n"));
    IncreaseTab(3);    
    while(true)
    {
        PrintTextWithTab(TEXT("Locking User PIN...\n"));
        IncreaseTab(3);
        for(DWORD i = 1;
            i < (MAX_USER_RETRY_COUNT + 1);
            i++)
        {
            /************************************************************************
            *   PIN-                      * 
            *  PIN-                                                   *
            ************************************************************************/
            PrintDword(i);
            PrintString(TEXT("Login User with wrong PIN...\n"));
            PrintTextWithTab(TEXT("C_Login:...\n"));
            ckResult = pFunctionList->C_Login(  hSession,
                                                CKU_USER,
                                                WRONG_USER_PIN,
                                                arraysize(WRONG_USER_PIN));
            if(ckResult == CKR_PIN_INCORRECT)
            {
                PrintTextWithTab(TEXT(" -> Wrong PIN\n"));
            }
            else
            {
                bResult = (ckResult == CKR_OK);
                if( !bResult )
                {
                    PrintTextWithTab(TEXT(" -> Failed\n"));
                    break;
                }
                else
                {
                    PrintTextWithTab(TEXT(" -> OK\n"));
                }
            }
        }
        DecreaseTab(3);

        /************************************************************************
        *                          *
        ************************************************************************/
        PrintTextWithTab(TEXT("Unlocking User PIN...\n"));
        IncreaseTab(3);
        PrintTextWithTab(TEXT("Login SO:...\n"));
        PrintTextWithTab(TEXT("C_Login:...\n"));
        ckResult = pFunctionList->C_Login(  hSession,
                                            CKU_SO,
                                            SO_PIN,
                                            arraysize(SO_PIN));

        bResult = (ckResult == CKR_OK);
        if(!bResult)
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));
        
        /************************************************************************
        *  PIN-                                    *
        ************************************************************************/
        PrintTextWithTab(TEXT("Unlock User PIN...\n"));
        PrintTextWithTab(TEXT("C_EX_UnblockUserPIN:...\n"));
        ckResult = pFunctionListExtended->C_EX_UnblockUserPIN(hSession);
        bResult = (ckResult == CKR_OK);
        if( !bResult )
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));

        /************************************************************************
        *                                                   *
        ************************************************************************/
        PrintTextWithTab(TEXT("Logout SO:...\n"));
        PrintTextWithTab(TEXT("C_Logout:...\n"));
        ckResult = pFunctionList->C_Logout(hSession);

        bResult = (ckResult == CKR_OK);
        if(!bResult)
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));
        DecreaseTab(3);
        break;
    }

    DecreaseTab(3);
    if(bResult)
    {
        PrintTextWithTab(TEXT("Extended PIN function test has been completed successfully!\n"));
    }
    else
    {
        PrintTextWithTab(TEXT("Extended PIN function test failed!\n"));
    }
    return bResult;
}

/************************************************************************
* main()                                                                *
************************************************************************/
int main(int argc, char* argv[])
{    
    HMODULE                         hModulePKCS11              = NULL_PTR; //    PKCS#11
    CK_SESSION_HANDLE               hSession                   = NULL_PTR; //        
    
    CK_FUNCTION_LIST_PTR            pFunctionList              = NULL_PTR; //     PKCS#11,    CK_FUNCTION_LIST
    CK_C_GetFunctionList            pfnGetFunctionList         = NULL_PTR; //    C_GetFunctionList
        
    CK_FUNCTION_LIST_EXTENDED_PTR   pFunctionListExtended      = NULL_PTR; //      PKCS#11,    CK_FUNCTION_LIST_EXTENDED    
    CK_C_EX_GetFunctionListExtended pfnGetFunctionListExtended = NULL_PTR; //    C_EX_GetFunctionListExtended
        
    bool                            bResult                    = false;    //       
    CK_RV                           ckResult                   = CKR_OK;   //      .               

    bool                            bIsRutokenECP              = false;    //       .     
    
    CK_SLOT_ID_PTR                  aSlots                     = NULL_PTR; //     
    CK_ULONG                        ulSlotCount                = 0;        //     


    CK_RUTOKEN_INIT_PARAM           initInfo_st;                           //    CK_RUTOKEN_INIT_PARAM,      C_EX_InitToken   

    PrintTextWithTab(TEXT("Initializing module...\n"));
    IncreaseTab(3);
    while(true)
    {
        /**********************************************************************
        *  1:  .                                        *      
        **********************************************************************/        
        PrintTextWithTab(TEXT("Loading library "));
        PrintString(PKCS11_LIBRARY_NAME);
        PrintString(TEXT("...\n"));
        hModulePKCS11 = LoadLibrary(PKCS11_LIBRARY_NAME);
        bResult = (hModulePKCS11 != NULL_PTR);
        if (!bResult)
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));

        /**********************************************************************
        *  2:              * 
        *           PKCS#11.                                *      
        **********************************************************************/    
        PrintTextWithTab(TEXT("Getting GetFunctionList Function...\n"));
        pfnGetFunctionList = (CK_C_GetFunctionList)GetProcAddress(    hModulePKCS11,
                                                                    "C_GetFunctionList");
        bResult = (pfnGetFunctionList != NULL_PTR);
        if (!bResult)
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));
        
        /**********************************************************************
        *  3:              *
        *            PKCS#11.                     *
        **********************************************************************/
        PrintTextWithTab(TEXT("Getting GetFunctionListExtended Function...\n"));        
        pfnGetFunctionListExtended = (CK_C_EX_GetFunctionListExtended)GetProcAddress(    hModulePKCS11,
                                                                                        "C_EX_GetFunctionListExtended"    );
        bResult = (pfnGetFunctionListExtended != NULL_PTR);
        if (!bResult)
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));

        /**********************************************************************
        *  4:                        *  
		*         PKCS#11.                                           *      
        **********************************************************************/
        PrintTextWithTab(TEXT("Getting function list...\n"));
        ckResult = pfnGetFunctionList(&pFunctionList);
        bResult = (ckResult == CKR_OK);
        if (!bResult)
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));

        /**********************************************************************
        *  5:              *
        *         PKCS#11.                                           *
        **********************************************************************/
        PrintTextWithTab(TEXT("Getting extended function list...\n"));
        ckResult = pfnGetFunctionListExtended(&pFunctionListExtended);
        bResult = (ckResult == CKR_OK);
        if (!bResult)
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));

        PrintTextWithTab(TEXT("C_Initialize:...\n"));

        /**********************************************************************
        *  6:  .                                 *      
        **********************************************************************/
        ckResult = pFunctionList->C_Initialize(NULL_PTR);
        bResult = (ckResult == CKR_OK);
        if(!bResult)
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));

        /**********************************************************************
        *  7:    c  .             *      
        **********************************************************************/
        PrintTextWithTab(TEXT("Getting connected slots list. Step 1...\n"));
        ckResult = pFunctionList->C_GetSlotList(    CK_TRUE,
                                                    NULL_PTR,
                                                    &ulSlotCount ); 
        bResult = (ckResult == CKR_OK);
        if(!bResult)
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));

        if (ulSlotCount == 0)
        {
            PrintTextWithTab(TEXT("Error: no Rutoken available!\n"));
            bResult = false;
            break;
        }

        aSlots = new CK_SLOT_ID[ulSlotCount];
        ::ZeroMemory( aSlots,
                      (ulSlotCount * sizeof(CK_SLOT_ID)));

        PrintTextWithTab(TEXT("Getting connected slots list. Step 2...\n"));
        ckResult = pFunctionList->C_GetSlotList(    CK_TRUE,
                                                    aSlots,
                                                    &ulSlotCount); 
        bResult = (ckResult == CKR_OK);
        if( !bResult )
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));

        PrintTextWithTab(TEXT("Slots available: "));
        PrintDword(ulSlotCount);
        PrintString(TEXT("\n"));

        /**********************************************************************
        *  8:   .                                     *      
        **********************************************************************/    
        PrintTextWithTab(TEXT("Determining token type:...\n"));    
        bResult = GetTokenClass(    hModulePKCS11,
                                    aSlots[0],
                                    bIsRutokenECP,
                                    ckResult );
        if(!bResult)
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));
        
        DecreaseTab(3);
        PrintTextWithTab(TEXT("Initialization is completing...\n"));    
        
        /**********************************************************************
        *  9:    CK_RUTOKEN_INIT_PARAM.              *      
        **********************************************************************/    
        PrintTextWithTab(TEXT("C_EX_InitToken:...\n"));
        ::ZeroMemory(    &initInfo_st,
                        sizeof(CK_RUTOKEN_INIT_PARAM) );

        initInfo_st.ulSizeofThisStructure = sizeof(CK_RUTOKEN_INIT_PARAM);
        initInfo_st.UseRepairMode         = 0;
        initInfo_st.pNewAdminPin          = SO_PIN;
        initInfo_st.ulNewAdminPinLen      = sizeof(SO_PIN);
        initInfo_st.pNewUserPin           = NEW_USER_PIN;
        initInfo_st.ulNewUserPinLen       = sizeof(NEW_USER_PIN);
        initInfo_st.ulMinAdminPinLen      = bIsRutokenECP ? 6 : 1;
        initInfo_st.ulMinUserPinLen       = bIsRutokenECP ? 6 : 1;
        initInfo_st.ChangeUserPINPolicy   = (TOKEN_FLAGS_ADMIN_CHANGE_USER_PIN | TOKEN_FLAGS_USER_CHANGE_USER_PIN);
        initInfo_st.ulMaxAdminRetryCount  = MAX_ADMIN_RETRY_COUNT;
        initInfo_st.ulMaxUserRetryCount   = MAX_USER_RETRY_COUNT;
        initInfo_st.pTokenLabel           = TOKEN_STD_LABEL;
        initInfo_st.ulLabelLen            = sizeof(TOKEN_STD_LABEL);

        /**********************************************************************
        *  10:  .                                     *
        **********************************************************************/
        ckResult = pFunctionListExtended->C_EX_InitToken(   aSlots[0],
                                                            SO_PIN,
                                                            arraysize(SO_PIN),
                                                            &initInfo_st );
        bResult = (ckResult == CKR_OK);
        if(!bResult)
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        
        /**********************************************************************
        *  11:      .                    * 
        **********************************************************************/
        PrintTextWithTab(TEXT("C_OpenSession:...\n"));
        ckResult = pFunctionList->C_OpenSession(    aSlots[0],
                                                    CKF_SERIAL_SESSION | CKF_RW_SESSION, 
                                                    NULL_PTR,
                                                    NULL_PTR,
                                                    &hSession );
        bResult = (ckResult == CKR_OK);
        if( !bResult )
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));
            
        /**********************************************************************
        *  12:    / PIN-         *
        *         .                                               *
        **********************************************************************/
        bResult = PerformExtendedPINTest(   hSession,
                                            pFunctionList,
                                            pFunctionListExtended,
                                            ckResult );
        if(!bResult)
        {
            break;
        }

        /**********************************************************************
        *  13:     .            *        
        **********************************************************************/
        PrintTextWithTab(TEXT("Login User...\n"));
        PrintTextWithTab(TEXT("C_Login:...\n"));
        ckResult = pFunctionList->C_Login(  hSession,
                                            CKU_USER,
                                            NEW_USER_PIN,
                                            arraysize(NEW_USER_PIN) );
        bResult = (ckResult == CKR_OK);
        if( !bResult )
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));

        /**********************************************************************
        *  14:  ""  .                              *        
        **********************************************************************/
        PrintTextWithTab(TEXT("Change Token Name...\n"));
        PrintTextWithTab(TEXT("C_EX_SetTokenName:...\n"));
        ckResult = pFunctionListExtended->C_EX_SetTokenName( hSession,
                                                             TOKEN_LONG_LABEL,
                                                             arraysize(TOKEN_LONG_LABEL) );
        bResult = (ckResult == CKR_OK);
        if( !bResult )
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));

        /**********************************************************************
        *  15:     .                *
        **********************************************************************/
        bResult = PrintExtendedTokenInfo( aSlots[0],
                                          pFunctionListExtended,
                                          ckResult);
        if(!bResult)
        {
            break;
        }

        /**********************************************************************
        *  16:  PIN-   .               *
        **********************************************************************/
        PrintTextWithTab(TEXT("Change User PIN to default...\n"));
        PrintTextWithTab(TEXT("C_SetPIN:...\n"));
        ckResult = pFunctionList->C_SetPIN( hSession,
                                            NEW_USER_PIN,
                                            arraysize(NEW_USER_PIN),
                                            USER_PIN,
                                            arraysize(USER_PIN));
        bResult = (ckResult == CKR_OK);
        if( !bResult )
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));

        break;
    }

    /**********************************************************************
    *  17:                         *
    *           PKCS#11.                                      *
    **********************************************************************/
    CK_RV ckTempResult = CKR_OK;
    Free(   hModulePKCS11,
            aSlots[0],
            hSession,
            pFunctionList,
            ckTempResult);

    if(aSlots)
    {
        delete[] aSlots;
        aSlots = NULL_PTR;
    }

    if(!bResult)
    {
        PrintLastError( TEXT("Some error occurred."),
                        ckResult);
    }
    else
    {
        PrintDone();
    }
    return bResult;
}

