/************************************************************************
* Rutoken                                                               *
* Copyright (C) Aktiv Co. 2003 - 2011                                   *
*  :  http://www.rutoken.ru                          *
*  :    http://www.rutoken.ru/hotline/download/drivers/*
*  : http://www.rutoken.ru/hotline/                 *
*-----------------------------------------------------------------------*
*    Rutoken    PKCS#11   C++    *
*-----------------------------------------------------------------------*
*                 *
*  :                                                            *
*  -    Rutoken    ;        *
*  -     PKCS#11;                         *
*  -     ;                           *
*  -     ;                       *
*  -     .                  *
*-----------------------------------------------------------------------*
*               *
*   PKCS#11                                                 *
************************************************************************/

/************************************************************************
*   PKCS11_Common.h.                                      *
*  PKCS11_Common.h  **     :  *
*  -   :                                     *
      - PKCS#11;                                                        *
	  -  /;                                      *  
*     -  Win32 API;                                              * 
*  -  DEMO-;                                             *  
*  -  DEMO PIN- ;                                *
*  -  ,    .      *
************************************************************************/
    

#include "PKCS11_Common.h"


/************************************************************************
*  :                                                *
*  -  ;                                                          *
*  -  ;                                              *
*  -  .                                         *
************************************************************************/
bool PrintSlotInfo( IN  CK_FUNCTION_LIST_PTR pFunctionList,  //     PKCS#11,    CK_FUNCTION_LIST
                    IN  CK_SLOT_ID           slotID,         //  
                    OUT CK_RV&               ckResult        //  .     ,   PKCS#11
                  )
{
    bool                  bResult          = false;    //           
    
    CK_SLOT_INFO          slotInfo;                    //    CK_SLOT_INFO    
    CK_TOKEN_INFO         tokenInfo;                   //    CK_TOKEN_INFO    
    CK_MECHANISM_INFO     mechInfo;                    //    CK_MECHANISM_INFO    

    CK_MECHANISM_TYPE_PTR aMechanisms      = NULL_PTR; //    ,  
    CK_ULONG              ulMechanismCount = 0;        //     

    PrintTextWithTab(TEXT("Printing slot info...\n"));
    IncreaseTab(3);    
    while(true)
    {
        /************************************************************************
        *                                               *
        ************************************************************************/
        ::ZeroMemory(&slotInfo,
                     sizeof(CK_SLOT_INFO));

        PrintTextWithTab(TEXT("C_GetSlotInfo...\n"));
        ckResult = pFunctionList->C_GetSlotInfo(slotID,
                                                &slotInfo);
        bResult = (ckResult == CKR_OK);
        if (!bResult)
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));

        /************************************************************************
        *                                            *
        ************************************************************************/
        PrintBufferAsCharString(TEXT("Slot description:  "),
                                slotInfo.slotDescription,
                                sizeof(slotInfo.slotDescription));

        PrintBufferAsCharString(TEXT("Manufacturer:      "),
                                slotInfo.manufacturerID,
                                sizeof(slotInfo.manufacturerID));
        
        PrintDwordWithString(TEXT("Flags:             "),
                             slotInfo.flags);

        PrintDotSeparatedByteValue(TEXT("Hardware Ver:      "),
                                   slotInfo.hardwareVersion.major,
                                   slotInfo.hardwareVersion.minor);

        PrintDotSeparatedByteValue(TEXT("Firmware Ver:      "),
                                   slotInfo.firmwareVersion.major,
                                   slotInfo.firmwareVersion.minor);

        if( slotInfo.flags & CKF_TOKEN_PRESENT )
        {            
            ::ZeroMemory(&tokenInfo,
                         sizeof(CK_TOKEN_INFO));

            /************************************************************************
            *                                            *
            ************************************************************************/
            PrintTextWithTab(TEXT("C_GetTokenInfo...\n"));
            ckResult = pFunctionList->C_GetTokenInfo(slotID,
                                                     &tokenInfo);
            bResult = (ckResult == CKR_OK);
            if (!bResult)
            {
                PrintTextWithTab(TEXT(" -> Failed\n"));
                break;
            }
            PrintTextWithTab(TEXT(" -> OK\n"));
            
            /************************************************************************
            *                                            *
            ************************************************************************/
            PrintBufferAsCharString(TEXT("Token label:                "),
                                    tokenInfo.label,
                                    sizeof(tokenInfo.label));

            PrintBufferAsCharString(TEXT("Manufacturer:               "),
                                    tokenInfo.manufacturerID,
                                    sizeof(tokenInfo.manufacturerID));

            PrintBufferAsCharString(TEXT("Token model:                "),
                                    tokenInfo.model,
                                    sizeof(tokenInfo.model));

            PrintBufferAsCharString(TEXT("Token #:                    "),
                                    tokenInfo.serialNumber,
                                    sizeof(tokenInfo.serialNumber));

            PrintDwordWithString(TEXT("Flags:                      "), tokenInfo.flags );        
            PrintDwordWithString(TEXT("Max session count:          "), tokenInfo.ulMaxSessionCount );
            PrintDwordWithString(TEXT("Current session count:      "), tokenInfo.ulSessionCount );
            PrintDwordWithString(TEXT("Max RW session count:       "), tokenInfo.ulMaxRwSessionCount );
            PrintDwordWithString(TEXT("Current RW session count:   "), tokenInfo.ulRwSessionCount );
            PrintDwordWithString(TEXT("Max PIN length:             "), tokenInfo.ulMaxPinLen );
            PrintDwordWithString(TEXT("Min PIN length:             "), tokenInfo.ulMinPinLen );        
            PrintDwordWithString(TEXT("Total public memory:        "), tokenInfo.ulTotalPublicMemory );
            PrintDwordWithString(TEXT("Free public memory:         "), tokenInfo.ulFreePublicMemory );
            PrintDwordWithString(TEXT("Total private memory:       "), tokenInfo.ulTotalPrivateMemory );
            PrintDwordWithString(TEXT("Free private memory:        "), tokenInfo.ulFreePrivateMemory );


            PrintDotSeparatedByteValue(TEXT("Hardware Ver:      "),
                                       tokenInfo.hardwareVersion.major,
                                       tokenInfo.hardwareVersion.minor);

            PrintDotSeparatedByteValue(TEXT("Firmware Ver:      "),
                                       tokenInfo.firmwareVersion.major,
                                       tokenInfo.firmwareVersion.minor);

            PrintBufferAsCharString(TEXT("Timer #:                    "),
                                    tokenInfo.utcTime,
                                    sizeof(tokenInfo.utcTime));

            /************************************************************************
            *                                               *
            ************************************************************************/
            ulMechanismCount = 0;
            PrintTextWithTab(TEXT("C_GetMechanismList step 1...\n"));
            ckResult = pFunctionList->C_GetMechanismList( slotID,
                                                          NULL_PTR,
                                                          &ulMechanismCount);
            bResult = (ckResult == CKR_OK);
            if (!bResult)
            {
                PrintTextWithTab(TEXT(" -> Failed\n"));
                break;
            }
            PrintTextWithTab(TEXT(" -> OK\n"));
            
            aMechanisms = new CK_MECHANISM_TYPE[ulMechanismCount];
            ::ZeroMemory( aMechanisms,
                          (sizeof(CK_MECHANISM_TYPE) * ulMechanismCount));

            PrintTextWithTab(TEXT("C_GetMechanismList step 2...\n"));
            ckResult = pFunctionList->C_GetMechanismList( slotID,
                                                          aMechanisms,
                                                          &ulMechanismCount);
            bResult = (ckResult == CKR_OK);
            if (!bResult)
            {
                PrintTextWithTab(TEXT(" -> Failed\n"));
                break;
            }
            PrintTextWithTab(TEXT(" -> OK\n"));            
            
            for(DWORD i = 0;
                i < ulMechanismCount;
                i++)
            {
                PrintDwordWithString(TEXT("Mechanism info "), i);
                IncreaseTab(3);                
                ::ZeroMemory( &mechInfo,
                              sizeof(CK_MECHANISM_INFO) );

                /************************************************************************
                *                                            *
                ************************************************************************/
                PrintTextWithTab(TEXT("C_GetMechanismInfo...\n"));
                ckResult = pFunctionList->C_GetMechanismInfo( slotID,
                                                              aMechanisms[i],
                                                              &mechInfo);
                bResult = (ckResult == CKR_OK);
                if (!bResult)
                {
                    PrintTextWithTab(TEXT(" -> Failed\n"));
                    DecreaseTab(3);
                    break;
                }
                PrintTextWithTab(TEXT(" -> OK\n"));
                
                /************************************************************************
                *                                         *
                ************************************************************************/
                PrintDwordWithString(TEXT("Mechanism type:    "), aMechanisms[i] );
                PrintDwordWithString(TEXT("Min key size:      "), mechInfo.ulMinKeySize );        
                PrintDwordWithString(TEXT("Max key size:      "), mechInfo.ulMaxKeySize );    
                PrintDwordWithString(TEXT("Mechanism flags:   "), mechInfo.flags );    
                DecreaseTab(3);
            } 
        }
        break;
    }

    if(aMechanisms)
    {
        delete[] aMechanisms;
        aMechanisms = NULL_PTR;
    }
    DecreaseTab(3);
    return bResult;
}

/************************************************************************
*                                        *
************************************************************************/
void PrintLibraryInfo(IN CK_INFO_PTR pInfo  //     CK_INFO     PKCS#11 
                      )
{
    PrintTextWithTab(TEXT("Printing info...\n"));
    IncreaseTab(3);
    PrintDotSeparatedByteValue( TEXT("PKCS#11 Ver:         "),
                                pInfo->cryptokiVersion.major,
                                pInfo->cryptokiVersion.minor);

    PrintBufferAsCharString(TEXT("Manufacturer:        "),
                            pInfo->manufacturerID,
                            sizeof(pInfo->manufacturerID));

    PrintDwordWithString(TEXT("Flags:               "),
                         pInfo->flags );

    PrintBufferAsCharString(TEXT("Library description: "),
                            pInfo->libraryDescription,
                            sizeof(pInfo->libraryDescription));
    DecreaseTab(3);
};

/************************************************************************
* main()                                                                *
************************************************************************/
int main(int argc, char* argv[])
{
    HMODULE               hModulePKCS11        = NULL_PTR; //    PKCS#11
    CK_SESSION_HANDLE     hSession             = NULL_PTR; //        

    CK_FUNCTION_LIST_PTR  pFunctionList        = NULL_PTR; //     PKCS#11,    CK_FUNCTION_LIST
    CK_C_GetFunctionList  pfnGetFunctionList   = NULL_PTR; //    C_GetFunctionList
    
    bool                  bResult              = false;    //       
    CK_RV                 ckResult             = CKR_OK;   //          
    
    CK_SLOT_ID_PTR        aAllSlots            = NULL_PTR; //       
    CK_ULONG              ulAllSlotCount       = 0;        //       

    CK_SLOT_ID_PTR        aSlotWithToken       = NULL_PTR; //        
    CK_ULONG              ulSlotWithTokenCount = 0;        //        
    
    CK_INFO               libraryInfo;                     //    CK_INFO     
    
    IncreaseTab(3);
    while(true)
    {
        /**********************************************************************
        *  1:  .                                        *      
        **********************************************************************/
        PrintTextWithTab(TEXT("Loading library "));
        PrintString(PKCS11_LIBRARY_NAME);
        PrintString(TEXT("...\n"));
        hModulePKCS11 = LoadLibrary(PKCS11_LIBRARY_NAME);
        bResult = (hModulePKCS11 != NULL_PTR);
        if (!bResult)
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));

        /**********************************************************************
        *  2:              * 
        *         .                                                  *      
        **********************************************************************/        
        PrintTextWithTab(TEXT("Getting GetFunctionListFunction...\n"));
        pfnGetFunctionList = (CK_C_GetFunctionList)GetProcAddress( hModulePKCS11,
                                                                   "C_GetFunctionList");
        bResult = (pfnGetFunctionList != NULL_PTR);
        if (!bResult)
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));

        /**********************************************************************
        *  3:      .                 *      
        **********************************************************************/        
        PrintTextWithTab(TEXT("Getting function list...\n"));
        ckResult = pfnGetFunctionList(&pFunctionList);
        bResult = (ckResult == CKR_OK);
        if (!bResult)
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));

        /**********************************************************************
        *  4:  .                                 *      
        **********************************************************************/
        PrintTextWithTab(TEXT("C_Initialize:...\n"));
        ckResult = pFunctionList->C_Initialize(NULL_PTR);
        bResult = (ckResult == CKR_OK);
        if(!bResult)
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));

        PrintTextWithTab(TEXT("C_GetInfo:...\n"));


        /**********************************************************************
        *  5:    .                            *      
        **********************************************************************/
        ::ZeroMemory(&libraryInfo,
                     sizeof(CK_INFO));

        ckResult = pFunctionList->C_GetInfo(&libraryInfo);
        bResult = (ckResult == CKR_OK);
        if(!bResult)
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));

        /**********************************************************************
        *  6:    .                         *      
        **********************************************************************/        
        PrintLibraryInfo(&libraryInfo);
        
        /**********************************************************************
        *  7:     .                       *      
        **********************************************************************/        
        PrintTextWithTab(TEXT("Getting all slots list. Step 1...\n"));
        ckResult = pFunctionList->C_GetSlotList(    CK_FALSE,
                                                    NULL_PTR,
                                                    &ulAllSlotCount    ); 
        bResult = (ckResult == CKR_OK);
        if( !bResult )
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));

        aAllSlots = new CK_SLOT_ID[ulAllSlotCount];
        ::ZeroMemory( aAllSlots,
                      (ulAllSlotCount * sizeof(CK_SLOT_ID)));

        PrintTextWithTab(TEXT("Getting all slots list. Step 2...\n"));
        ckResult = pFunctionList->C_GetSlotList(    CK_FALSE,
                                                    aAllSlots,
                                                    &ulAllSlotCount    ); 
        bResult = (ckResult == CKR_OK);
        if( !bResult )
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));
        
        /**********************************************************************
        *  8:      .             *
        **********************************************************************/
        PrintTextWithTab(TEXT("Getting connected slots list. Step 1...\n"));
        ckResult = pFunctionList->C_GetSlotList(    CK_TRUE,
                                                    NULL_PTR,
                                                    &ulSlotWithTokenCount    ); 
        bResult = (ckResult == CKR_OK);
        if( !bResult )
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));

        aSlotWithToken = new CK_SLOT_ID[ulSlotWithTokenCount];
        ::ZeroMemory( aSlotWithToken,
                     (ulSlotWithTokenCount * sizeof(CK_SLOT_ID)));

        PrintTextWithTab(TEXT("Getting connected slots list. Step 2...\n"));
        ckResult = pFunctionList->C_GetSlotList(    CK_TRUE,
                                                    aSlotWithToken,
                                                    &ulSlotWithTokenCount    ); 
        bResult = (ckResult == CKR_OK);
        if( !bResult )
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));

        PrintDwordWithString(TEXT("Number of all slots:       "),    ulAllSlotCount);
        PrintDwordWithString(TEXT("Number of connected slots: "),    ulSlotWithTokenCount);

        for(DWORD i = 0;
            i < ulAllSlotCount;
            i++)
        {
            /************************************************************************
            *  9:  :                                         *
            *         -  ;                                                   *
            *         -   ;                                     *
            *         -   .                                *
            ************************************************************************/
            PrintDwordWithString(TEXT("Slot number: "), i);
            bResult = PrintSlotInfo(    pFunctionList,
                                        aAllSlots[i],
                                        ckResult);
            if(!bResult)
            {
                bResult;
            }
        }
        break;
    }

    if(aSlotWithToken)
    {
        delete[] aSlotWithToken;
        aSlotWithToken = NULL_PTR;
    }
    if(aAllSlots)
    {
        delete[] aAllSlots;
        aAllSlots = NULL_PTR;
    }

    /**********************************************************************
    *  10:  .                              *
    **********************************************************************/
    if(pFunctionList)
    {
        pFunctionList->C_Finalize(NULL_PTR);
        pFunctionList = NULL_PTR;
    }

    /**********************************************************************
    *  11:    .                             *
    **********************************************************************/
    if(hModulePKCS11)
    {
        FreeLibrary(hModulePKCS11);
        hModulePKCS11 = NULL_PTR;
    }
    if(!bResult)
    {
        PrintLastError( TEXT("Some error occurred."),
                        ckResult);
    }
    else
    {
        PrintDone();
    }
    return bResult;
}

