/************************************************************************
* Rutoken                                                               *
* Copyright (C) Aktiv Co. 2003 - 2011                                   *
*  :  http://www.rutoken.ru                          *
*  :    http://www.rutoken.ru/hotline/download/drivers/*
*  : http://www.rutoken.ru/hotline/                 *
*-----------------------------------------------------------------------*
*    Rutoken    PKCS#11   C++    *
*-----------------------------------------------------------------------*
*   /    RSA:           *
*  -    Rutoken    ;        *
*  -     Rutoken (Login);         *
*  -     ;                       *
*  -     ;                        * 
*  -      Rutoken (Logout)                *
*        Rutoken.                                   *
*-----------------------------------------------------------------------*
*   ,    Rutoken   *
* (.  ..\Create)                                                 *
************************************************************************/

/************************************************************************
*   PKCS11_Common.h.                                      *
*  PKCS11_Common.h  **     :  *
*  -   :                                     *
      - PKCS#11;                                                        *
	  -  /;                                      *  
*     -  Win32 API;                                              * 
*  -  DEMO-;                                             *  
*  -  DEMO PIN- ;                                *
*  -  ,    .      *
************************************************************************/
    

#include "PKCS11_Common.h"


/*     */
CK_OBJECT_CLASS ocPubKey  = CKO_PUBLIC_KEY;
CK_OBJECT_CLASS ocPrivKey = CKO_PRIVATE_KEY;

/*    */
CK_BYTE pbtData[] = {   0x00, 0x01, 0x02,0x03, 0x04, 0x05, 0x06, 0x07,
                        0x00, 0x01, 0x02,0x03, 0x04, 0x05, 0x06, 0x07,
                        0x00, 0x01, 0x02,0x03, 0x04, 0x05, 0x06, 0x07,
                        0x00, 0x01, 0x02,0x03, 0x04, 0x05, 0x06, 0x07 };

/*    SHA-1 */
CK_MECHANISM    ckmSHA1Mech = { CKM_SHA_1, 0, 0  };

/*   /    RSA */
CK_MECHANISM    ckmRSASigVerMech = { CKM_RSA_PKCS, NULL_PTR, 0  };

/*    RSA    */
CK_ATTRIBUTE attrRSASignerTempl[] = 
{
    { CKA_ID,    (CK_VOID_PTR)RSA_KEY_PAIR_ID, (CK_ULONG)sizeof(RSA_KEY_PAIR_ID)-1},
    { CKA_CLASS, &ocPrivKey,                   sizeof(ocPrivKey)}    
};

/*    RSA     */
CK_ATTRIBUTE attrRSAVerifierTempl[] = 
{
    { CKA_ID,    (CK_VOID_PTR)RSA_KEY_PAIR_ID, (CK_ULONG)sizeof(RSA_KEY_PAIR_ID)-1},
    { CKA_CLASS, &ocPubKey,                    sizeof(ocPubKey)}
};


/************************************************************************
* main()                                                                *
************************************************************************/
int main(void)
{
    HMODULE               hModulePKCS11      = NULL_PTR; //    PKCS#11
    CK_SLOT_ID            slotID             = 0;        //  ,       
    CK_SESSION_HANDLE     hSession           = NULL_PTR; //   
    CK_FUNCTION_LIST_PTR  pFunctionList      = NULL_PTR; //     PKCS#11,    CK_FUNCTION_LIST
    
    bool                  bResult            = false;    //       
    CK_RV                 ckResult           = CKR_OK;   //      .                    

    bool                  bIsRutokenECP      = false;    //       .                     
        
    CK_BYTE_PTR           pbtSignedData     = NULL_PTR;  //   ,      
    CK_ULONG              ulSignedDataSize  = 0;         //   ,    ,  
    

    while(true)
    {
        /**********************************************************************
        *  1:        PKCS#11.  *
        **********************************************************************/
        bResult = Init( PKCS11_LIBRARY_NAME,
                        CKF_SERIAL_SESSION | CKF_RW_SESSION,
                        CKU_USER,
                        (const CK_UTF8CHAR_PTR)USER_PIN,
                        (CK_ULONG)sizeof(USER_PIN),
                        hModulePKCS11,
                        slotID,
                        hSession,
                        pFunctionList,
                        bIsRutokenECP,
                        ckResult );
        if(!bResult)
        {
            break;
        }

        /**********************************************************************
        *  2:      RSA.                   *
        **********************************************************************/        
        bResult = SignData( hSession,
                            pFunctionList,
                            attrRSASignerTempl,
                            arraysize(attrRSASignerTempl),
                            &ckmSHA1Mech,                            
                            &ckmRSASigVerMech,
                            pbtData,
                            arraysize(pbtData),
                            pbtSignedData,
                            ulSignedDataSize,
                            ckResult);
        if(!bResult)
        {
            break;
        }

        /**********************************************************************
        *  3:       RSA.          *
        **********************************************************************/            
        bResult = VerifySignature(  hSession,
                                    pFunctionList,
                                    attrRSAVerifierTempl,
                                    arraysize(attrRSAVerifierTempl),
                                    &ckmSHA1Mech,
                                    &ckmRSASigVerMech,
                                    pbtData,
                                    arraysize(pbtData),
                                    pbtSignedData,
                                    ulSignedDataSize,
                                    ckResult );
        if(!bResult)
        {
            break;
        }
        break;
    }

    /**********************************************************************
    *  4:                          *
    *          PKCS#11.                                       *
    **********************************************************************/
    CK_RV ckTempResult = CKR_OK;
    Free(   hModulePKCS11,
            slotID,
            hSession,
            pFunctionList,
            ckTempResult);

    if(pbtSignedData)
    {
        delete[] pbtSignedData;
        pbtSignedData = NULL_PTR;
    }

    if(!bResult)
    {
        PrintLastError( TEXT("Some error occurred."),
                        ckResult);
    }
    else
    {
        PrintDone();
    }
    return bResult;
}