/************************************************************************
* Rutoken                                                               *
* Copyright (C) Aktiv Co. 2003 - 2011                                   *
*  :  http://www.rutoken.ru                          *
*  :    http://www.rutoken.ru/hotline/download/drivers/*
*  : http://www.rutoken.ru/hotline/                 *
*-----------------------------------------------------------------------*
*    Rutoken    PKCS#11   C++    *
*-----------------------------------------------------------------------*
*   /    :          *
*  -    Rutoken    ;        *
*  -     Rutoken (Login);         *
*  -     ;                       *
*  -     ;                        * 
*  -      Rutoken (Logout)                *
*        Rutoken.                                   *
*-----------------------------------------------------------------------*
*   ,    Rutoken   *
* (.  ..\Create_GOST)                                            *
************************************************************************/

/************************************************************************
*   PKCS11_Common.h.                                      *
*  PKCS11_Common.h  **     :  *
*  -   :                                     *
      - PKCS#11;                                                        *
	  -  /;                                      *  
*     -  Win32 API;                                              * 
*  -  DEMO-;                                             *  
*  -  DEMO PIN- ;                                *
*  -  ,    .      *
************************************************************************/
    

#include "PKCS11_Common.h"


/*     */
CK_OBJECT_CLASS ocPubKey  = CKO_PUBLIC_KEY;
CK_OBJECT_CLASS ocPrivKey = CKO_PRIVATE_KEY;

/*    */
CK_BYTE pbtData[] = {   0x00, 0x01, 0x02,0x03, 0x04, 0x05, 0x06, 0x07,
                        0x00, 0x01, 0x02,0x03, 0x04, 0x05, 0x06, 0x07,
                        0x00, 0x01, 0x02,0x03, 0x04, 0x05, 0x06, 0x07,
                        0x00, 0x01, 0x02,0x03, 0x04, 0x05, 0x06, 0x07 };

/*      34.11-94 */
CK_MECHANISM    ckmGOST34_11_94Mech = { CKM_GOSTR3411, NULL_PTR, 0  };

/*   /      34.10-2001 */
CK_MECHANISM    ckmGOST_34_10_2001SigVerMech = { CKM_GOSTR3410, NULL_PTR, 0  };

/*      34.10-2001     */
CK_ATTRIBUTE attrGOST34_10_2001VerifierTempl[] = 
{
    { CKA_ID,    (CK_VOID_PTR)GOST_KEY_PAIR_ID_1, (CK_ULONG)sizeof(GOST_KEY_PAIR_ID_1)-1},
    { CKA_CLASS, &ocPubKey,                       sizeof(ocPubKey)}
};

/*      34.10-2001    */
CK_ATTRIBUTE attrGOST34_10_2001SignerTempl[] = 
{
    { CKA_ID,    (CK_VOID_PTR)GOST_KEY_PAIR_ID_1, (CK_ULONG)sizeof(GOST_KEY_PAIR_ID_1)-1},
    { CKA_CLASS, &ocPrivKey,                      sizeof(ocPrivKey)}
};


/************************************************************************
* main()                                                                *
************************************************************************/
int main(void)
{
    HMODULE               hModulePKCS11      = NULL_PTR; //    PKCS#11
    CK_SLOT_ID            slotID             = 0;        //  ,       
    CK_SESSION_HANDLE     hSession           = NULL_PTR; //   
    CK_FUNCTION_LIST_PTR  pFunctionList      = NULL_PTR; //     PKCS#11,    CK_FUNCTION_LIST
    
    bool                  bResult            = false;    //       
    CK_RV                 ckResult           = CKR_OK;   //      .                    

    bool                  bIsRutokenECP      = false;    //       .                     
        
    CK_BYTE_PTR           pbtSignedData     = NULL_PTR;  //   ,      
    CK_ULONG              ulSignedDataSize  = 0;         //   ,    ,  
    

    while(true)
    {
        /**********************************************************************
        *  1:        PKCS#11.  *
        **********************************************************************/
        bResult = Init( PKCS11ECP_LIBRARY_NAME,
                        CKF_SERIAL_SESSION | CKF_RW_SESSION,
                        CKU_USER,
                        (const CK_UTF8CHAR_PTR)USER_PIN,
                        (CK_ULONG)sizeof(USER_PIN),
                        hModulePKCS11,
                        slotID,
                        hSession,
                        pFunctionList,
                        bIsRutokenECP,
                        ckResult );
        if(!bResult)
        {
            break;
        }

        if(!bIsRutokenECP)
        {
            PrintTextWithTab(TEXT("This test is not supported for current token type...\n"));
            break;
        }

        /**********************************************************************
        *  2:        34.10-2001.     *
        **********************************************************************/        
        bResult = SignData( hSession,
                            pFunctionList,
                            attrGOST34_10_2001SignerTempl,
                            arraysize(attrGOST34_10_2001SignerTempl),
                            &ckmGOST34_11_94Mech,
                            &ckmGOST_34_10_2001SigVerMech,
                            pbtData,
                            arraysize(pbtData),
                            pbtSignedData,
                            ulSignedDataSize,
                            ckResult);
        if(!bResult)
        {
            break;
        }

        /**********************************************************************
        *  3:                                *
        *            34.10-2001.                              * 
        **********************************************************************/        
        bResult = VerifySignature(  hSession,
                                    pFunctionList,
                                    attrGOST34_10_2001VerifierTempl,
                                    arraysize(attrGOST34_10_2001VerifierTempl),
                                    &ckmGOST34_11_94Mech,
                                    &ckmGOST_34_10_2001SigVerMech,
                                    pbtData,
                                    arraysize(pbtData),
                                    pbtSignedData,
                                    ulSignedDataSize,
                                    ckResult );
        if(!bResult)
        {
            break;
        }
        break;
    }

    /**********************************************************************
    *  4:                          *
    *          PKCS#11.                                       *
    **********************************************************************/
    CK_RV ckTempResult = CKR_OK;
    Free(   hModulePKCS11,
            slotID,
            hSession,
            pFunctionList,
            ckTempResult);

    if(pbtSignedData)
    {
        delete[] pbtSignedData;
        pbtSignedData = NULL_PTR;
    }

    if(!bResult)
    {
        PrintLastError( TEXT("Some error occurred."),
                        ckResult);
    }
    else
    {
        PrintDone();
    }
    return bResult;
}
