/************************************************************************
* Rutoken                                                               *
* Copyright (C) Aktiv Co. 2003 - 2011                                   *
*  :  http://www.rutoken.ru                          *
*  :    http://www.rutoken.ru/hotline/download/drivers/*
*  : http://www.rutoken.ru/hotline/                 *
*-----------------------------------------------------------------------*
*    Rutoken    PKCS#11   C++    *
*-----------------------------------------------------------------------*
*        :        *
*  -  ;                                          *
*  -    -  (                 *
*      );                                     *
*  -      -             *
*   (    ).                      *
*-----------------------------------------------------------------------*
*               *
*   PKCS#11                                                 *
************************************************************************/

/************************************************************************
*   PKCS11_Common.h.                                      *
*  PKCS11_Common.h  **     :  *
*  -   :                                     *
      - PKCS#11;                                                        *
	  -  /;                                      *  
*     -  Win32 API;                                              * 
*  -  DEMO-;                                             *  
*  -  DEMO PIN- ;                                *
*  -  ,    .      *
************************************************************************/
        

#include "PKCS11_Common.h"


/*  ,     - */
#define MONITORING_THREADS_NUMBER    3

/************************************************************************
*  ,                            *
*                                       *
************************************************************************/
typedef struct _MONITORING_THREADS_PARAMS
{  
    CK_FUNCTION_LIST_PTR    m_pFunctionList;
    CK_FLAGS                m_flags;
    DWORD                   m_dwThread_Number;
    
} MONITORING_THREADS_PARAMS, *PMONITORING_THREADS_PARAMS;


/************************************************************************
*  ,    .                           *
*      .               *
************************************************************************/
void Monitoring_Slots( IN void* param //      MONITORING_THREADS_PARAMS     
                      )
{
    bool                  bResult            = false;      //       

    HMODULE               hModulePKCS11      = NULL_PTR;   //    PKCS#11
    CK_FUNCTION_LIST_PTR  pFunctionList      = NULL_PTR;   //     PKCS#11,    CK_FUNCTION_LIST

    CK_SLOT_ID            slotID             = 0xFFFFFFFF; //  ,        
    CK_SLOT_INFO          slotInfo;                        //    CK_SLOT_INFO    

    CK_RV                 ckResult           = CKR_OK;     //      
    
    CK_FLAGS              ckFlags            = 0;          //     ,    C_WaitForSlotEvent
    DWORD                 dwThreadNumber     = 0;          //         
    

    /************************************************************************
    *      MONITORING_THREADS_PARAMS           *
    *                                           *
    ************************************************************************/
    PMONITORING_THREADS_PARAMS pMonitoring_Threads_Param = (PMONITORING_THREADS_PARAMS)param;

    pFunctionList    = pMonitoring_Threads_Param->m_pFunctionList;
    ckFlags          = pMonitoring_Threads_Param->m_flags; 
    dwThreadNumber   = pMonitoring_Threads_Param->m_dwThread_Number;
    
    while(true)
    {    
        /************************************************************************
        *     ,                  *
        * C_WaitForSlotEvent     ckFlags                 *   
        ************************************************************************/
        slotID = 0xFFFFFFFF;
        ckResult = pFunctionList->C_WaitForSlotEvent(   ckFlags,
                                                        &slotID,
                                                        NULL_PTR);    
        if(ckResult != CKR_OK)
        {
            if (ckResult == CKR_CRYPTOKI_NOT_INITIALIZED)
            {
                PrintString(TEXT("\nWork with PKCS#11 has been finished.\n"));    
                break;
            }
            else if (ckResult == CKR_NO_EVENT)
            {
                PrintTextWithTab(TEXT("No more slot events...\n"));
                break;
            }
            else
            {            
                bResult = false;
                PrintTextWithTab(TEXT(" -> Failed\n"));
                break;
            }
        }        
         
        ::ZeroMemory( &slotInfo, 
                      sizeof(CK_SLOT_INFO));        

        /************************************************************************
        *                                               *
        ************************************************************************/   
        ckResult = pFunctionList->C_GetSlotInfo( slotID,
                                                 &slotInfo);
        if(ckResult != CKR_OK)            
        {
            bResult = false;
            PrintTextWithTab(TEXT(" -> Failed\n"));
            break;
        }
           
        /************************************************************************
        *                         *
        ************************************************************************/
        PrintDwordWithString(TEXT("Monitoring thread: "), dwThreadNumber);
        PrintDwordWithString(TEXT("    Slot ID: "), slotID );
        if (slotInfo.flags & CKF_TOKEN_PRESENT)
        {
            PrintTextWithTab(TEXT("        Token has been attached!\n\n"));
        }
        else
        {
            PrintTextWithTab(TEXT("        Token has been detached!\n\n"));
        }    
    }
    
    PrintDwordWithString(TEXT("Exiting from thread: "), dwThreadNumber);
    PrintString(TEXT("\n"));
}

/************************************************************************
* main()                                                                *
************************************************************************/
int main(int argc, char* argv[])
{
    HMODULE               hModulePKCS11        = NULL_PTR;   //    PKCS#11    

    CK_FUNCTION_LIST_PTR  pFunctionList        = NULL_PTR;   //     PKCS#11,    CK_FUNCTION_LIST    
    CK_C_GetFunctionList  pfnGetFunctionList   = NULL_PTR;   //    C_GetFunctionList

    bool                  bResult              = false;      //       
    CK_RV                 ckResult             = CKR_OK;     //          

    CK_SLOT_ID            slotID               = 0xFFFFFFFF; //  ,        
    CK_SLOT_INFO          slotInfo;                          //    CK_SLOT_INFO    

    DWORD                 i                    = 0;          //  .   
        
    
    IncreaseTab(3);
    while(true)
    {
        /**********************************************************************
        *  1:  .                                        *      
        **********************************************************************/        
        PrintTextWithTab(TEXT("Loading library "));
        PrintString(PKCS11_LIBRARY_NAME);
        PrintString(TEXT("...\n"));
        hModulePKCS11 = LoadLibrary(PKCS11_LIBRARY_NAME);
        bResult = (hModulePKCS11 != NULL_PTR);
        if (!bResult)
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            DecreaseTab(3);
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));

        /**********************************************************************
        *  2:                          * 
        *           .                                    *      
        **********************************************************************/
        PrintTextWithTab(TEXT("Getting GetFunctionListFunction...\n"));
        pfnGetFunctionList = (CK_C_GetFunctionList)GetProcAddress(hModulePKCS11,
                                                                  "C_GetFunctionList");
        bResult = (pfnGetFunctionList != NULL_PTR);
        if (!bResult)
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            DecreaseTab(3);
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));

        /**********************************************************************
        *  3:      .                 *      
        **********************************************************************/
        PrintTextWithTab(TEXT("Getting function list...\n"));
        ckResult = pfnGetFunctionList(&pFunctionList);
        bResult = (ckResult == CKR_OK);
        if (!bResult)
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            DecreaseTab(3);
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));
        
        /**********************************************************************
        *  4:  .                                 *      
        **********************************************************************/
        PrintTextWithTab(TEXT("C_Initialize:...\n"));
        ckResult = pFunctionList->C_Initialize(NULL_PTR);
        bResult = (ckResult == CKR_OK);
        if(!bResult)
        {
            PrintTextWithTab(TEXT(" -> Failed\n"));
            DecreaseTab(3);
            break;
        }
        PrintTextWithTab(TEXT(" -> OK\n"));
        DecreaseTab(3);
            
        PrintTextWithTab(TEXT("\nPlease attach or detach Rutoken and press Enter...\n"));
        getchar();

        i = 1;
        while(true)
        {
            PrintDwordWithString(TEXT("Events counter: "), i);            
            
            /**********************************************************************
            *  5:                                     *
            *        (  ,   CKF_DONT_BLOCK ).        *
            **********************************************************************/
            PrintTextWithTab(TEXT("C_WaitForSlotEvent:...\n"));
            ckResult = pFunctionList->C_WaitForSlotEvent( CKF_DONT_BLOCK,
                                                          &slotID,
                                                          NULL_PTR ); 

            bResult = (   (ckResult == CKR_OK)
                       || (ckResult == CKR_NO_EVENT));
            if (!bResult)
            {
                PrintTextWithTab(TEXT(" -> Failed\n"));
                DecreaseTab(3);
                break;
            }
            PrintTextWithTab(TEXT(" -> OK\n"));

            if (ckResult == CKR_NO_EVENT )
            {
                PrintTextWithTab(TEXT("No more slots events...\n"));
                break;
            }            

            /**********************************************************************
            *  6:      .                   *      
            **********************************************************************/
            ::ZeroMemory( &slotInfo, 
                          sizeof(CK_SLOT_INFO));

            PrintTextWithTab(TEXT("C_GetSlotInfo...\n"));
            ckResult = pFunctionList->C_GetSlotInfo(slotID,
                                                    &slotInfo);
            bResult = (ckResult == CKR_OK);
            if (!bResult)
            {
                PrintTextWithTab(TEXT(" -> Failed\n"));
                break;
            }
            PrintTextWithTab(TEXT(" -> OK\n"));

            PrintDwordWithString(TEXT("Slot ID:           "),
                                 slotID);

            PrintBufferAsCharString(TEXT("Slot description:  "),
                                    slotInfo.slotDescription,
                                    sizeof(slotInfo.slotDescription));

            PrintBufferAsCharString(TEXT("Manufacturer:      "),
                                    slotInfo.manufacturerID,
                                    sizeof(slotInfo.manufacturerID));
            
            PrintDwordWithString(TEXT("Flags:             "),
                                 slotInfo.flags);

            PrintDotSeparatedByteValue(TEXT("Hardware Ver:      "),
                                       slotInfo.hardwareVersion.major,
                                       slotInfo.hardwareVersion.minor);

            PrintDotSeparatedByteValue(TEXT("Firmware Ver:      "),
                                       slotInfo.firmwareVersion.major,
                                       slotInfo.firmwareVersion.minor);  

            i++;
            PrintTextWithTab(TEXT("\n"));        
        }

        if(!bResult)
        {
            break;
        }

        PrintString(TEXT("\n\n\n"));

        /**********************************************************************
        *  7:   ,                 *
        *         - .                                          *          
        *                      *
        *        .          *        
        *          .                                      *
        ***********************************************************************/
        while(true)
        {    
            MONITORING_THREADS_PARAMS aThreads_With_Blocking[MONITORING_THREADS_NUMBER];

            for(i = 0;
                i < MONITORING_THREADS_NUMBER;
                i++ )
            {
                PrintString(TEXT("Starting monitoring thread number "));
                PrintDword(i);
                PrintString(TEXT("\n"));
                
                ::ZeroMemory( &aThreads_With_Blocking[i],
                              sizeof(MONITORING_THREADS_PARAMS));

                aThreads_With_Blocking[i].m_pFunctionList   = pFunctionList;
                aThreads_With_Blocking[i].m_flags           = 0;
                aThreads_With_Blocking[i].m_dwThread_Number = i;

                _beginthread(Monitoring_Slots,
                             0,
                             &aThreads_With_Blocking[i]);

            }
            PrintString(TEXT("\n\n\n"));
            PrintTextWithTab(TEXT("\nPlease attach or detach Rutoken or press Enter to exit.\n"));    
            
            getchar();     


            /**********************************************************************
            *  8:  .                               *
            **********************************************************************/        
            pFunctionList->C_Finalize(NULL_PTR);
            pFunctionList = NULL_PTR;

            break;
        }
        break;
    }
    
    if(    !bResult
        && pFunctionList)
    {
        pFunctionList->C_Finalize(NULL_PTR);
        pFunctionList = NULL_PTR;
    }

    /**********************************************************************
    *  9:    .                              *
    **********************************************************************/
    if(hModulePKCS11)
    {
        FreeLibrary(hModulePKCS11);
        hModulePKCS11 = NULL_PTR;
    }

    if(!bResult)
    {
        PrintLastError(TEXT("Some error occurred."),
                       ckResult);
    }
    else
    {
        PrintDone();
    }

    return bResult;
}

